import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import javax.microedition.io.*;
import java.io.*;

/**
 * The ClientGui class is the front-end GUI of the client application.
 * It is handling all the user interaction and setting up connections to
 * the Service Provider and the Supplicant.
 * The connections to the different components are set up by the inner thread classes
 * @author Hvard Holje
 * @since May 4, 2007
 */
public class ClientGui extends MIDlet implements CommandListener {
    private Command exitCommand; // The exit command
    private Command submitCommand;
    private Command connectCommand; // Connect to Service Provider
    private Command balanceCommand; //Get the account balance

    private Display display;    // The display for this MIDlet
    private Form aForm; // The start form
    private Form pinForm; //The pin form
    private Form banking; //The banking "service" form
    private TextField pin; //The input PIN field

    private InputStream is;
    private HttpConnection httpConn;

    private ServiceProviderClient spClient;
    private ConnectionThread connectionThread;
    private SupplicantClient supplicantClient;

    private String sessionID;
    private String URL = "socket://127.0.0.1:4035";

    /**
     * The constructor
     */
    public ClientGui() {
        exitCommand = new Command("Exit", Command.SCREEN, 1);
        submitCommand= new Command("Submit", Command.OK, 1);;
        connectCommand = new Command("Start auth", Command.OK, 2);
        balanceCommand = new Command("Get balance", Command.OK, 2);

        pin = new TextField("Pin:", "", 30, TextField.NUMERIC);

        aForm = new Form("Client application for SIM authentication");
        aForm.addCommand(exitCommand);
        aForm.addCommand(connectCommand);
        aForm.setCommandListener(this);

        pinForm = new Form("Please enter your PIN");
        pinForm.addCommand(exitCommand);
        pinForm.addCommand(submitCommand);

        banking = new Form("Hvard's mobile bank");
        banking.addCommand(exitCommand);
        banking.addCommand(balanceCommand);
        banking.setCommandListener(this);
    }

    /**
     * Standard method required on J2ME MIDlets
     * This is the method starting up the application from the AMS
     */
    public void startApp() {
        if (display == null) {
            display = Display.getDisplay(this);
        }
        aForm.append("Welcome!\n\nTo start the authentication procedure please" +
        		" choose the 'start auth' option in the menu. This will contact" +
        		" the Service Provider to redirect you to the local Supplicant." +
        		" The Supplicant will start running automatically when you are" +
        		" redirected from the Service Provider. \n\nThe next thing you " +
        		"will have to to is to enter your PIN when I'm asking for it");
        display.setCurrent(aForm);
    }

    public void pauseApp() {}

    public void destroyApp(boolean unconditional) {}

    /**
     * This is where the commands are handeled
     */
    public void commandAction(Command c, Displayable s) {
        if (c == exitCommand) {
            destroyApp(false);
            notifyDestroyed();
        }
        else if(c == submitCommand) {
        	String userPin = pin.getString();
        	supplicantClient = new SupplicantClient(display, userPin);
        	supplicantClient.start();
        }

        else if(c == connectCommand) {
            connectionThread = new ConnectionThread(this, display);
        	connectionThread.start();
        }

        else if(c == balanceCommand) {
        	spClient = new ServiceProviderClient(display);
        	spClient.start();

        }
    }

    /**
     * Changes the current display to the "PIN input form"
     * @param display
     */
    private void showPinForm(Display display) {
    	pinForm.append(pin);
    	pinForm.setCommandListener(this);
    	display.setCurrent(pinForm);
    }


    /**
     * The SupplicantClient class is setting up the connection to the local
     * Supplicant.
     * @author Hvard Holje
     * @since May 9, 2007
     *
     */
	class SupplicantClient extends Thread {
	    private Display display;
	    private String userPin;
	    private XmlParser xmlParser;
	    private UserSession session;
	    private String SA;

	    public SupplicantClient(Display display, String userPin) {
	    	this.display = display;
	    	this.userPin = userPin;
	    	xmlParser = new XmlParser();
	    }

	    public void run() {
	        try {

	           	writeToCurrentForm("Opening connection to: " + URL);

	        	SocketConnection client = (SocketConnection)Connector.open(URL);

	        	writeToCurrentForm("Connected to Supplicant!");

	        	InputStream is = client.openInputStream();
	        	OutputStream os = client.openOutputStream();

	        	//send PIN to Supplicant to active the authentication procedure
	        	String xml = xmlParser.createAuthenticationResponse(userPin);

	        	os.write(xml.trim().getBytes());
	        	os.write("\r\n".getBytes());
	        	os.flush();

	        	writeToCurrentForm("Please wait while Supplicant performs authentication..");

	        	//read server response
	        	StringBuffer sb = new StringBuffer();

	        	//loop forever, receiving data
	            while (true) {
	                int c = 0;

	                while (((c = is.read()) != '\n') && (c != -1)) {
	                    sb.append((char)c);
	                }

	                if (c == -1) {
	                    break;
	                }

	                session = xmlParser.getAuthenticationResponse(sb.toString());
	                SA = session.getSecurityAssociation();

	                if( (SA != null) && (SA != "") ) {
	                	writeToCurrentForm("Got answer from Supplicant: Authentication succeeded! SA=" + SA);
	                	ServiceProviderClient spClient = new ServiceProviderClient(display, SA);
	                	spClient.start();
	                }
	                else if( (session.getErrorMsg() != null) && (session.getErrorMsg() != "") ) {
	                	writeToCurrentForm("Got answer from Suplicant: " + session.getErrorMsg());

	                }
	                else {
	                	writeToCurrentForm("An unknown error occurred... You where NOT authenticated");
	                }
	            }

	            //close streams and connection
	        	is.close();
	        	os.close();
	        	client.close();

	        }
	        catch (Exception exp) {
	            exp.printStackTrace();
	        }
	    }

	    public void writeToCurrentForm(String msg) {
	    	System.out.println(msg); //debug

			Form currentForm = (Form) display.getCurrent();
		    if (currentForm != null){
		        currentForm.append(msg);

		    }
		}
	}


	/**
     * The ServiceProviderClient class is setting up a connection to the Service Provider,
     * when the user is auhenticated by the Supplicant.
     * @author Hvard Holje
     * @since May 9, 2007
     *
     */
    class ServiceProviderClient extends Thread {
        private Display display;
        private String SpUrl = "http://dellgeopos.item.ntnu.no:8080/ServiceProvider/servlet/Service";
        private String securityAssociation;

        public ServiceProviderClient(Display display, String securityAssociation) {
            this.display = display;
            this.securityAssociation = securityAssociation;
        }

        public ServiceProviderClient(Display display) {
        	this.display = display;
        }

        public void run() {
            try {

            	writeToCurrentForm("Opening connection to Service Provider.. ");

            	// Authorization is already granted, pass the sessionID to Service Provider
            	// and start using the provided service!
               	if (sessionID != null) {

               		httpConn = (HttpConnection) Connector.open(SpUrl);
                	writeToCurrentForm("Connection establised! ");
                	httpConn.setRequestMethod( HttpConnection.GET );
                	httpConn.setRequestProperty("Content-Type","text/plain" );

            		// Set sessionID in request
            		httpConn.setRequestProperty("Cookie", sessionID);

            		is = httpConn.openInputStream();

            		// read server response
                	StringBuffer sb = new StringBuffer();

                	// loop forever, receiving data
                    while (true) {
                    	int c = 0;

    	            	while (((c = is.read()) != '\n') && (c != -1)) {
    	            		sb.append((char)c);
    	                }

    	                if (c == -1) {
    	                    break;
    	                }
    	                writeToCurrentForm(sb.toString());
                    }

            	}

               	// sessionID doesn't exist, we have to request one
            	else {
            		if(securityAssociation != null) {
            			httpConn = (HttpConnection) Connector.open(SpUrl);
                    	writeToCurrentForm("Connection establised! ");
                    	httpConn.setRequestMethod( HttpConnection.GET );
                    	httpConn.setRequestProperty("Content-Type","text/plain" );

            			httpConn.setRequestProperty("SA", securityAssociation);

            			is = httpConn.openInputStream();

                	    String cookie = httpConn.getHeaderField("Set-Cookie");

                	    if (cookie != null) {
                	    	int semicolon = cookie.indexOf(';');
                	    	sessionID = cookie.substring(0, semicolon);

                	    	//debug
                	        writeToCurrentForm("Access granted!!!");

                	        //Show "Service form"
                	        showBankingForm(display);
                	    }

                	 /*	int length = (int)httpConn.getLength();
                	    byte[] raw = new byte[length];
                	    is.read(raw);
                	 */

            		}
            	}

            	//Closing inputStream and HttpConnection
            	is.close();
        	    httpConn.close();

            }
            catch (Exception exp) {
                exp.printStackTrace();
            }
        }

        public void writeToCurrentForm(String msg) {
        	System.out.println(msg); //debug

    		Form currentForm = (Form) display.getCurrent();
    	    if (currentForm != null){
    	        currentForm.append(msg);

    	    }
    	}

        private void showBankingForm(Display display) {
        	display.setCurrent(banking);
        }
    }


    /**
     * The ConnectionThread class connects to Service Provider, receives a redirect,
     * interprets and sets up a connection to the local Supplicant.
     * @author Hvard Holje
     * @since May 4, 2007
     */
    class ConnectionThread extends Thread {
        private Display display;
        private ClientGui midlet;
        private String SP_URL = "http://dellgeopos.item.ntnu.no:8080/ServiceProvider/servlet/Service";

        public ConnectionThread (ClientGui midlet, Display display) {
        	this.midlet = midlet;
        	this.display = display;
        }

        public void run() {
            try {
            	// Step 1: Contact Service Provider and get redirect url
              	writeToCurrentForm("Opening connection to Service Provider.. ");

             	httpConn = (HttpConnection) Connector.open(SP_URL);
               	httpConn.setRequestMethod( HttpConnection.GET );
            	httpConn.setRequestProperty("Content-Type","text/plain" );

            	is = httpConn.openInputStream();

            	// Get response code from server
            	int code = httpConn.getResponseCode();

            	if(code == HttpConnection.HTTP_MOVED_TEMP) { //Redirect from server
            		String redirectURL = httpConn.getHeaderField("Location");

            		//Replace 'http' with 'socket'
            		URL = "socket://" + redirectURL.substring(7, redirectURL.length());

            		writeToCurrentForm("You have been redirected to " + URL);
            		is.close();
            		httpConn.close();

            		// Now we've got a connection to the Supplicant and is ready
            		// to start the authentication procedure.
            		// First we must show the PIN form
            		midlet.showPinForm(display);

             	}
            	else { // return error messsage, i.e. server is down
            		writeToCurrentForm("An error has occured!\nCould not contact Service Provider.\nPlease chech your network settings and try again..");
            	}

            }
            catch (Exception exp) {
                exp.printStackTrace();
            }
        }

        public void writeToCurrentForm(String msg) {
        	System.out.println(msg); //debug

    		Form currentForm = (Form) display.getCurrent();
    	    if (currentForm != null){
    	        currentForm.append(msg);

    	    }
    	}
    }

}

