package SIMCardApplet;

import javacard.framework.*;

/**
 * This applet simulates a GSM SIM card, with the following steps:
 * - Verify CHV (PIN) must be done to get access to other services on PIN
 * - Get IMSI
 * - Run A3 alghoritm with a Challenge(RAND)
 *
 * Runs in the Java Card environment (Java_card_kit_2.2.1)
 * @author Hvard Holje
 * @since 12.feb.2006
 */

public class SIM extends Applet {

  //standard APDU input offset values
  public final static byte THIS_CLA = (byte)0xA0;

  //INS value for dummy response (echo)
  public final static byte INS_DUMMY = (byte)0x10;

  //INS value for verify CHV according to GSM 11.11
  public final static byte VERIFY_CHV = (byte)0x20;

  //INS value for retrieving IMSI
  public final static byte GET_IMSI = (byte)0x30;

  //INS value for RUN GSM ALGORITHM according to GSM 11.11
  public final static byte RUN_GSM_ALGORITHM = (byte)0x88;

  //signal that the PIN verification failed
  final static short SW_VERIFICATION_FAILED = 0x6300;

  //The user PIN (0000)
  private final static byte[] pinCode = {(byte)0x00,(byte)0x00,(byte)0x00,(byte)0x00};

  //maximum number of incorrect tries before the PIN is blocked
  final static byte PIN_TRY_LIMIT =(byte)0x03;

  //maximum size PIN
  final static byte MAX_PIN_SIZE =(byte)0x04;

  //For offset computations
  final static short START = 0;

  private OwnerPIN cardPIN;
  private UserPIN userPin;


  //sendResponse
  private byte[] echoBytes;
  private static final short LENGTH_ECHO_BYTES = 256;

  /**
   * Constructor.
   * Only this class's install method can create the applet object.
   */
  public SIM(byte[] installationData, short offset, byte length) {

	  cardPIN = new OwnerPIN(PIN_TRY_LIMIT, MAX_PIN_SIZE);
	  //pin - the byte array containing the new PIN value
	  //offset - the starting offset in the pin array
	  //length - the length of the new PIN.
	  cardPIN.update(pinCode,(short)0, MAX_PIN_SIZE);
	  echoBytes = new byte[LENGTH_ECHO_BYTES];
	  register();
  }

  /**
   * Installs this applet.
   * @param byteArray the array containing installation parameters
   * @param offset the starting offset in byteArray
   * @param length the length in bytes of the parameter data in byteArray
   */
  public static void install(byte[] byteArray, short offset, byte length) {
    new SIM(byteArray, offset, length);
  }

  /**
   * Implementation  of the standard method for processing an incoming APDU.
   * @param apdu the incoming APDU
   * @exception ISOException with ISO 7816-4 response bytes
   */
  public void process(APDU apdu) {
    byte buffer[] = apdu.getBuffer();

    //Examines the first two bytes of the APDU header, the CLA byte and INS byte.
	//If the value of the CLA byte is 0 and the value of the INS byte is 0xA4,
	//it indicates that this is the header of a SELECT APDU command.
	//In this case, the process method returns control to the JCRE:
	if ((buffer[ISO7816.OFFSET_CLA] == 0) &&
	   (buffer[ISO7816.OFFSET_INS] == (byte)(0xA4)) ) {
	  return;
	}

    if (buffer[ISO7816.OFFSET_CLA] == THIS_CLA) {

    	switch (buffer[ISO7816.OFFSET_INS])	{
        	case VERIFY_CHV:
        		verifyCHV(apdu);
        		break;

        	case GET_IMSI:
        		getImsi(apdu);
        		break;

        	case RUN_GSM_ALGORITHM:
        		runGSMAlgorithm(apdu);
        		break;

        	case INS_DUMMY:
        		sendResponse(apdu);
        		break;

        	default:
        		ISOException.throwIt(ISO7816.SW_INS_NOT_SUPPORTED);
      }
    }
  }


  /**
   *
   * This method verifies the PIN in the APDU buffer against a PIN established
   * when the applet was installed. If the values match, the method returns
   * a the response code '9000'. 
  */
  private void verifyCHV(APDU apdu) {

	  //Obtains a reference to the APDU buffer (which contains the message)
      //Only the first five APDU header bytes are available in the APDU buffer.
      //CLA, INS, P1, P2, and P3 bytes
      //(Byte P3 denotes the Lc byte, if the command has optional data)
	  //Lc byte denotes the number of bytes in the
	  //data field of the command APDU
	  
	  byte[] buffer = apdu.getBuffer();
  	  byte pinLength = buffer[ISO7816.OFFSET_LC];
  	

  	//retrieve the PIN data for validation.
  	//(indicate that this APDU has incoming data
  	//and receive data starting from the offset
  	//ISO7816.OFFSET_CDATA following the 5 header bytes).
  	short bytesRead = apdu.setIncomingAndReceive();
  	short echoOffset = (short)0;

  	//Copies the incoming bytes to echoBytes
  	while ( bytesRead > 0 ) {
        Util.arrayCopyNonAtomic(buffer, ISO7816.OFFSET_CDATA, echoBytes, echoOffset, bytesRead);
        echoOffset += bytesRead;
        bytesRead = apdu.receiveBytes(ISO7816.OFFSET_CDATA);
    }

  	// check pin
  	// the PIN data is read into the APDU buffer
  	// at the offset ISO7816.OFFSET_CDATA
  	// the PIN data length = byteRead

  	if (cardPIN.check(apdu.getBuffer(), (short)5, (byte)4) == false) {
          ISOException.throwIt(SW_VERIFICATION_FAILED);
      } else {

       	  //1. short bOff - the offset into APDU buffer
    	  //2. short len - the bytelength of the data to send
    	  //Send nothing but SW1 and SW2, which is '90 00'
    	  apdu.setOutgoingAndSend((short)0, (short) 0);

      }
    } // end of validate method

  
  /**
   * Simulates the get IMSI function and returns an imaginary  IMSI
   * @param apdu
   */
  public void getImsi(APDU apdu) {
	  
	  byte IMSI[] =  {(byte)1,(byte)2,(byte)3,(byte)4,(byte)5,
				(byte)6,(byte)7,(byte)8,(byte)9,(byte)8,
				(byte)7,(byte)6,(byte)5,(byte)4,(byte)3};

	  apdu.setOutgoing();

	  apdu.setOutgoingLength( (short) (IMSI.length) );

	  apdu.sendBytesLong( IMSI, (short) 0, (short)(IMSI.length) );
  }

  /**
   * Simulates the run GSM Algorithm and returns SRES and Kc.
   * (In real life the response is covered by a MAC)
   * @param apdu
   */
  public void runGSMAlgorithm(APDU apdu) {

	  //Byte 1-4 = SRES
	  //Byte 5-12 is Kc according to GSM 11.11 version 5.0.0
	  byte SRES_KC[] =  {(byte)4,(byte)4,(byte)4,(byte)4, //SRES
			  			 (byte)8,(byte)8,(byte)8,(byte)8, //Kc
			  			 (byte)8,(byte)8,(byte)8,(byte)8}; //Kc

	  apdu.setOutgoing();

	  apdu.setOutgoingLength( (short) (SRES_KC.length) );

	  apdu.sendBytesLong( SRES_KC, (short) 0, (short)SRES_KC.length );
  }


  /**
   * This method echo's the received bytes
   * @param apdu
   */
  protected void sendResponse(APDU apdu) {
	  	byte buffer[] = apdu.getBuffer();

		short bytesRead = apdu.setIncomingAndReceive();
		short echoOffset = (short)0;

		while ( bytesRead > 0 ) {
          Util.arrayCopyNonAtomic(buffer, ISO7816.OFFSET_CDATA, echoBytes, echoOffset, bytesRead);
          echoOffset += bytesRead;
          bytesRead = apdu.receiveBytes(ISO7816.OFFSET_CDATA);
      }

      apdu.setOutgoing();
      apdu.setOutgoingLength( (short) (echoOffset + 5) );

      // echo header
      apdu.sendBytes( (short)0, (short) 5);
      // echo data
      apdu.sendBytesLong( echoBytes, (short) 0, echoOffset );
  }



}