package com;

import javax.microedition.apdu.*;
import java.io.*;
import javax.microedition.io.*;
import core.SIMSupplicant;

/**
 * See ETSI GSM 11.11 and ETSI EG 201 220 V1.4.0 for more details.
 * Defines the Card Application Identifier (AID) for GSM
 *
 * 0xa0:0x0:0x0:0x0:0x62 is the fixed part of the apdu
 *
 * The AID of the installer applet is 0xa0:0x00:0x00:0x00:0x62:0x03:0x01:0x08:0x01
 * The AID of the applet is:          0xa:0x0:0x0:0x0:0x62:0x3:0x1:0x1
 * Format: <protocol>:<slotID>;target=<AID>
 * (for apdu it is: apdu:<slotID>;target=<AID> )
 *
 * Because a handset may support more that one smart card, SATSA defines a special
 * property to discover the available smart card slots. This property, called
 * microedition.smartcardslots, can be queried by calling the method System.getProperty()
 * which returns a comma-delimited list of available slots
 * Java card supports two smart cards according: 0H,1H (0 and 1 with Hotswap functionality..)
 *
 * @author Hvard Holje
 * @since March 20, 2007
 *
 */

public class SIMProxy extends Thread  {

	// AID of the applet to be installed, wrapped inside the command APDU
	// Applet AID (0xa:0x0:0x0:0x0:0x62:0x3:0x1:0x1)
	// 4 first bytes are mandatory header (CLA, INS, P1 and P2)
	// Byte #5 is Lc which is 8
	private final String AID = "apdu:0;target=a0.00.00.00.62.03.01.0c.05.01";

	// The connection object
	private APDUConnection apduConnection;

	// Reference to the SIMSupplicant
	private SIMSupplicant simSupplicant;

	private byte[] apdu;
	private int type;

	private byte[] CMD_VERIFY_CHV = null;
	private byte[] CMD_GET_IMSI = null;
	private byte[] CMD_RUN_GSM_ALG = null;

	private boolean finish = false;

	/**
	 * Constructor 1
	 * @param simSupplicant
	 * @param type
	 */
	public SIMProxy(SIMSupplicant simSupplicant, int type) {
		this.simSupplicant = simSupplicant;
		this.type = type;
	}

	/**
	 * Constructor 2
	 * @param apdu
	 */
	public SIMProxy(SIMSupplicant simSupplicant, byte[] apdu, int type) {
		this.simSupplicant = simSupplicant;
		this.apdu = apdu;
		this.type = type;
	}

	/**
	 * @param type
	 */
	public void setType(int type) {
		this.type = type;
	}

	/**
	 * Sets the CMD_VERIFY_CHV Command APDU
	 * @param CMD_VERIFY_CHV
	 */
	public void setVerifyPinCmd(byte[] CMD_VERIFY_CHV) {
		this.CMD_VERIFY_CHV = CMD_VERIFY_CHV;
	}

	/**
	 * Sets the finish flag to close the APDU Connection
	 */
	public void setFinish() {
		finish = true;
	}

	/**
	 * Sets the CMD_GET_IMSI Command APDU and reset the old ones
	 * @param CMD_GET_IMSI
	 */
	public void setGetImsiCmd(byte[] CMD_GET_IMSI) {
		this.CMD_GET_IMSI = CMD_GET_IMSI;
	}

	/**
	 * Sets the CMD_RUN_GSM_ALG Command APDU and reset the old ones
	 * @param CMD_RUN_GSM_ALG
	 */
	public void setRunGsmAlgCmd(byte[] CMD_GET_IMSI) {
		this.CMD_RUN_GSM_ALG = CMD_GET_IMSI;

	}


	/**
	 * All I/O handling is executing in a Thread
	 */
	public synchronized void run() {

		byte[] response = null;

		try {
			// Connect to SIM
			connect();

			while(!finish) {

				if(CMD_VERIFY_CHV != null) {
					// Get the response from the SIM card applet
					apdu = CMD_VERIFY_CHV;
					response = sendAPDUCommand(apdu);

					System.out.println("if(CMD_VERIFY_CHV != null)");

					// Handle the response from SIM applet
					simSupplicant.handleSIMResponse(type, byteArrayHexToString(response));

					// Reset the Command APDU
					CMD_VERIFY_CHV = null;
				}

				if(CMD_GET_IMSI != null) {
					// Get the response from the SIM card applet
					apdu = CMD_GET_IMSI;
					response = sendAPDUCommand(apdu);

					System.out.println("if(CMD_GET_IMSI != null) {");

					// Handle the response from SIM applet
					simSupplicant.handleSIMResponse(type, byteArrayToString(response));

					// Reset the Command APDU
					this.CMD_GET_IMSI = null;
				}

				if(CMD_RUN_GSM_ALG != null) {
					// Get the response from the SIM card applet
					apdu = CMD_RUN_GSM_ALG;
					response = sendAPDUCommand(apdu);

					// Handle the response from SIM applet
					simSupplicant.handleSIMResponse(type, byteArrayToString(response));

					// Reset the Command APDU
					this.CMD_RUN_GSM_ALG = null;
				}
			}

			// Close the connection if we are finish
			if(finish) {
				System.out.println("Finish! Closing apduConnection");
				apduConnection.close();
			}
		}

		catch(SecurityException se) {
			System.out.println(se);
		}

		catch(NullPointerException ne) {
			System.out.println(ne);
		}

		catch(IOException ioe){
			System.out.println(ioe.toString());
		}
	}

	/**
	 * Setting up connection to the SIM
	 */
	public void connect() {
		//Debug - Retrieves the supported slots
		//System.out.println(System.getProperty("microedition.smartcardslots").toString());
		System.out.println("setting up APDU connection...");

		try {
			apduConnection = (APDUConnection)Connector.open(AID);
		}

		catch (ConnectionNotFoundException ce) {
	        System.out.println("Connection to server not found...." + ce.toString());
		}

		catch(IOException ioe){
			System.out.println(ioe.toString());
			try { apduConnection.close(); } catch (Throwable t) {}
		}
	}

	/**
	 * Helper method that sends an APDU Command to the SIM
	 * @param byte[] request
	 * @return byte[] response
	 */
	private byte[] sendAPDUCommand(byte[] request) {
		byte[] response = null;

		//Debug
		System.out.println("**Debug-exchangeAPDU: **" + bytesToHex(request));

		try {
			response = apduConnection.exchangeAPDU(request);
		}
		catch(IOException ioe){
			System.out.println(ioe.toString());
			try { apduConnection.close(); } catch (Throwable t) {}
		}

		return response;
	}

	/**
	 * Helper method that converts the received byte array
	 * to a readable String.
	 * @param response
	 * @return
	 */
	private String byteArrayToString(byte[] response) {
		StringBuffer buff = new StringBuffer();
		if(response.length > 0)  {
			for(int i = 0; i < response.length; i++) {
				buff.append(response[i]);
			}
		}
		return buff.toString();
	}

	/**
	 *
	 * Helper method that interprets the APDU response from SIM
	 * Converts HEX values from the byte Array to a String.
	 *
	 * @param response
	 * @return
	 */
	public String byteArrayHexToString(byte[] response) {

		StringBuffer buff = new StringBuffer();
		if(response.length > 0)  {
			for(int i = 0; i < response.length; i++) {
				System.out.print(byteToHex(response[i]));
				buff.append(byteToHex(response[i]));
			}
		}

		return buff.toString();
	}


	/**
	* Convenience method to convert a byte array to a hex string.
	*
	* @param data the byte[] to convert
	* @return String the converted byte[]
	*/
	public String bytesToHex(byte[] data) {
		StringBuffer buf = new StringBuffer();
		for ( int i = 0; i < data.length; i++ ) {
			buf.append( byteToHex(data[i]) );
		}
		return(buf.toString());
	}

	/**
	* Convenience method to convert a byte to a hex string.
	*
	* @param data the byte to convert
	* @return String the converted byte
	*/
	public String byteToHex(byte data)
	{
	StringBuffer buf = new StringBuffer();
	buf.append(toHexChar((data>>>4)&0x0F));
	buf.append(toHexChar(data&0x0F));
	return buf.toString();
	}

	/**
	* Convenience method to convert an int to a hex char.
	*
	* @param i the int to convert
	* @return char the converted char
	*/
	public char toHexChar(int i) {
		if ((0 <= i) && (i <= 9 ))
			return (char)('0' + i);
		else
			return (char)('a' + (i-10));
	}


} //END class SIMProxy
