package core;

import java.io.*;
import base.*;

import com.IdpProxy;

/**
 * This class binds all the classes together by parsing inputStreams and interpreting data
 * @author Hvard Holje
 */
public class Controller {
    private ServiceSupplicant midlet;
    private AuthenticationRequest authReq;
    private XmlParser xml;
    private AuthenticationSession authSession;
    private SIMSupplicant simSupplicant;
    private IdpProxy idp;
    private int type;

     /**
     * @param midlet ServiceSupplicant
     */
	public Controller(ServiceSupplicant midlet) {
	    this.midlet = midlet;

	    // Initializes the authentication object, which keeps the
	    // state of the authentication procedure
	    authSession = new AuthenticationSession();

	    // Initializes the SIMSupplicant
	    simSupplicant = new SIMSupplicant(this);

	}

	/**
	 * Catch incoming data from clientApp and handle them properly
	 * @param is InputStream
	 * @param type int
	 */
	public void handleInputStream(String input, int type) {

	    switch(type) {
	    	case 1: // Validate PIN

	        	if ( (input != null) && (input != "") ) {
		        	xml = new XmlParser();
		            authReq = xml.getAuthenticationRequest(input);

		            String pin = authReq.getPin();
		            simSupplicant.validatePin(pin);
		            break;
	        	}

	        case 2: // Get IMSI
	        	simSupplicant.getIMSI();
	        	break;

	        case 3: // Run GSM Algorithm
	        	simSupplicant.getSRES();

	        case 4: // Close connection to SIM applet
	        	simSupplicant.finish();
	        default:
	            break;

		}

	}

	/**
	 * This method interprets the response from the SIMSupplicant,
	 * sets the state and calls the appropriate handler for next step.
	 * @param type
	 * @param response
	 */
	public void handleSIMResponse(int type, String response) {

		switch(type) {

			case 1: // PIN Validated

				if(response.equalsIgnoreCase("9000")) { //PIN validated
					midlet.print("PIN validated!");
					authSession.setPinValidated();

					// Go to next step
					handleInputStream("",2);
				}

				else { // Should handle possible error situations here
					// The user should have maximum 3 attempts to present a valid PIN
					midlet.print("PIN was not validated..");
				}

				break;

			case 2: // Get IMSI
				if( (response != null) && (response != "") ) {
					midlet.print("IMSI received: " + response);
					authSession.setIMSI(response);

					// Go to next step
					handleInputStream("",3);
				} else {
					midlet.print("Could not receive IMSI..");
				}
				break;

			case 3: //Run GSM Algorithm
				String SRES = response.substring(0,4);
				String Kc = response.substring(4,12);
				authSession.setSRES(SRES);
				midlet.print("SRES received: " + SRES);

				authSession.setKc(Kc);
				midlet.print("Kc received: " + Kc);

				// Now is time to authenticate the client against the
				// Identity Provider
	   		    authenticateClient(authSession);

				break;

			default:
				break;
		}
	}

	/**
	 * Takes care of the auth response from IDP.
	 * @param String response
	 */
	public void handleIdpResponse(String response) {
		AuthenticationResponse authResponse = new AuthenticationResponse();
		authResponse.setSecurityAssociation(response);

		// Create the xml AuthenticationResponse according to the XML Schema
		String xmlResponse = createAuthResponseXml(authResponse);

		// Send AuthenticationResponse to the client
		midlet.sendResponseToClientApp(type, xmlResponse);

		// Done! Close Apdu Connection
		midlet.print("Done! Closing APDUConnection..");
		handleInputStream("", 4);
	}


	/**
	 * Requests authentication from the Identity Provider
	 * @param auth
	 * @return AuthenticationResponse authResponse
	 */
	private void authenticateClient(AuthenticationSession authSession) {
		idp = new IdpProxy(this,authSession.getSRES());
		idp.start();
	}


	/**
	 * Creates the xml response to send to the client when it's authenticated
	 * @param authResponse
	 * @return
	 */
	private String createAuthResponseXml(AuthenticationResponse authResponse) {
		XmlParser xmlParser = new XmlParser();
		return xmlParser.createAuthenticationResponse(authResponse);
	}

	/**
	 * Converts the InputStream to String.
	 * @param is InputStream
	 * @return String InputStream
	 */
	private String iSToString(InputStream is) {
	    String res = null;
	    byte[] enBolk = null;
	    byte[] alle = null;
	    try {
			enBolk = new byte[512];
			alle = new byte[0];//need a value != null for arraycopy
			int antallIBolk = 0;
			int totaltLest = 0;
			while ((antallIBolk = is.read(enBolk)) != -1 ){
				byte[] tmp = new byte[totaltLest + antallIBolk];//Have to be big enough
				System.arraycopy(alle,0,tmp,0,totaltLest);//transmits the data to a larger array
				alle = tmp;
				System.arraycopy(enBolk,0,alle,totaltLest,antallIBolk);//copies the last bit into the array
				totaltLest += antallIBolk;
			}
			res = new String(alle).trim();
			System.out.println("--------------"+res);

	    } catch (IOException e) {
	        System.out.println(e);
	    }
	    return res;
	}
}