package core;

import com.SIMProxy;

/**
 * This class handles all SIM communication. 
 * It initializes a new thread that creates the APDU connection to the SIM 
 * @author Hvard Holje
 * @since 4. May 2007
 */
public class SIMSupplicant {
	
	private Controller controller;
	private SIMProxy simProxy;
	private String pin;
	
	/**
	 * Constructor 
	 * @param controller
	 * @param pin
	 */
	public SIMSupplicant(Controller controller, String pin) {
		this.controller = controller;
		this.pin = pin;
	}
	
	/**
	 * Constructor
	 * @param controller
	 */
	public SIMSupplicant(Controller controller) {
		this.controller = controller;
	}
	
	/**
	 * @param pin
	 */
	public void setPin(String pin) {
		this.pin = pin;
	}
	
	/**
	 * Calls the request handler with type=1 (PIN)
	 */
	public void validatePin(String pin) {
		setPin(pin);
		handleSIMRequest(1);
	}
	
	/**
	 * Calls the request handler with type=2 (GET_IMSI)
	 */
	public void getIMSI() {
		handleSIMRequest(2);
	}
	
	/**
	 * Calls the request handler with type=3 (RUN_GSM_ALGORITHM)
	 */
	public void getSRES() {
		handleSIMRequest(3);
	}
	
	public void finish() {
		simProxy.setFinish();
	}
	
	/**
	 * This method handles the SIM request and creates proper Command APDU
	 * @param type
	 */
	public void handleSIMRequest(int type) {
		switch(type) {
			/*
			 * Validate PIN
			 */
			case 1: 
				
				//Converts pin to byte array
				byte[] apdu = strToByteArray(pin);
				
				//Creates the Command APDU for VERIFY_CHV
				byte[] CMD_VERIFY_CHV = {	
				  (byte)0xA0, (byte)0x20,
				  (byte)0x00, (byte)0x01, (byte)0x04,
				  apdu[0],apdu[1],apdu[2],apdu[3],(byte)0x7F};
				
				//Initializes the SIMProxy thread that verifies PIN against the SIM
				simProxy = new SIMProxy(this, type);
				simProxy.setVerifyPinCmd(CMD_VERIFY_CHV);
				simProxy.start();
				break;
			
			/*
			 * Get IMSI
			 */	
			case 2:
				
				//Creates the Command APDU for Get IMSI
				byte[] CMD_GET_IMSI = {	
					  (byte)0xA0, (byte)0x30,
					  (byte)0x00, (byte)0x01,
					  (byte)0x01, (byte)0x00,(byte)0x7F};
				
				//Sets type and correct Command APDU for GET_IMSI
				simProxy.setType(type);
				simProxy.setGetImsiCmd(CMD_GET_IMSI);
				break;
				
			/*
			 * Run GSM Algorithm
			 */	
			case 3: 
				byte[] CMD_RUN_GSM_ALG = {	
					  (byte)0xA0,(byte)0x88, 			//CLA, INS
					  (byte)0x00,(byte)0x00,(byte)0x10,	//P1, P2, Lc=10hex (16)
					  (byte)0x09,(byte)0x09,(byte)0x09,	//"RAND" 16 bytes	
					  (byte)0x09,(byte)0x09,(byte)0x09, //"RAND" 
					  (byte)0x09,(byte)0x09,(byte)0x09, //"RAND" 
					  (byte)0x09,(byte)0x09,(byte)0x09, //"RAND" 
					  (byte)0x09,(byte)0x09,(byte)0x09, //"RAND" 
					  (byte)0x09,						//"RAND" 
					  (byte)0x7F};						//Le
	
				//Sets type and correct Command APDU for GET_IMSI
				simProxy.setType(type);
				simProxy.setRunGsmAlgCmd(CMD_RUN_GSM_ALG);

				break;
		
			default:
				break;
		}
	} //END handleSIMRequest()
	
	
	
	/**
	 * This method handles the SIM request and creates proper Command APDU
	 * @param type
	 */
	public void handleSIMRequest_old(int type) {
		switch(type) {
			/*
			 * Validate PIN
			 */
			case 1: 
				
				//Converts pin to byte array
				byte[] apdu = strToByteArray(pin);
				
				//Creates the Command APDU for VERIFY_CHV
				byte[] CMD_VERIFY_CHV = {	
				  (byte)0xA0, (byte)0x20,
				  (byte)0x00, (byte)0x01, (byte)0x04,
				  apdu[0],apdu[1],apdu[2],apdu[3],(byte)0x7F};
				
				//Initializes the SIMProxy thread that verifies PIN against the SIM
				simProxy = new SIMProxy(this, CMD_VERIFY_CHV, type);
				simProxy.start();
				break;
			
			/*
			 * Get IMSI
			 */	
			case 2:
				
				//Creates the Command APDU for Get IMSI
				byte[] CMD_GET_IMSI = {	
					  (byte)0xA0, (byte)0x30,
					  (byte)0x00, (byte)0x01,
					  (byte)0x01, (byte)0x00,(byte)0x7F};
				
				//Initializes the SIMProxy thread that verifies PIN against the SIM
				simProxy = new SIMProxy(this, CMD_GET_IMSI, type);
				simProxy.start();
				break;
				
			/*
			 * Run GSM Algorithm
			 */	
			case 3: 
				byte[] CMD_RUN_GSM_ALG = {	
					  (byte)0xA0,(byte)0x88, 			//CLA, INS
					  (byte)0x00,(byte)0x00,(byte)0x10,	//P1, P2, Lc=10hex (16)
					  (byte)0x09,(byte)0x09,(byte)0x09,	//"RAND" 16 bytes	
					  (byte)0x09,(byte)0x09,(byte)0x09, //"RAND" 
					  (byte)0x09,(byte)0x09,(byte)0x09, //"RAND" 
					  (byte)0x09,(byte)0x09,(byte)0x09, //"RAND" 
					  (byte)0x09,(byte)0x09,(byte)0x09, //"RAND" 
					  (byte)0x09,						//"RAND" 
					  (byte)0x7F};						//Le
	
				//Initializes the SIMProxy thread that verifies PIN against the SIM
				simProxy = new SIMProxy(this, CMD_RUN_GSM_ALG, type);
				simProxy.start();
				break;
			
			default:
				break;
		}
	} //END handleSIMRequest()
	
	
	/**
	 * Handles the response from SIM. Parses the input and calls
	 * the appropriate handler in the Controller. 
	 * @param type
	 */
	public void handleSIMResponse(int type, String response) {
		
		switch(type) {
			case 1: //PIN
				controller.handleSIMResponse(type, response);
				break;

			case 2: //IMSI
				//Retrieve IMSI from Response APDU
				//IMSI is the first 15 bytes, the rest is overhead from APDU Response
				String imsi = response.substring(0, 15);
								
				controller.handleSIMResponse(type, imsi);
				break;
			
			case 3: //SRES from RUN_GSM_ALGORITHM
				//Retrieve SRES and Kc from Response APDU
				//SRES is byte 1-4
				//Kc is byte 5-12
	
				String SRES_KC = response.substring(0,12);								
				controller.handleSIMResponse(type, SRES_KC);
				break;
			
			default:
				break;
		}
		
	} //END handleSIMResponse()
	
	
	/**
	 * Converting the input String to a byte array
	 * @param String pin
	 * @return byte[] pin
	 */
	public byte[] strToByteArray(String pin) {
	
		byte[] pinArray = new byte[4];
		try {
	  	
	    	int tmp = 0;
	    	pinArray = new byte[4];
	    	
	    	for(int i = 0; i < pin.length(); i++) {
	    		tmp = Integer.parseInt(pin.substring(i, i+1));
	    		pinArray[i] = (byte)tmp;
	    	}
		} 
		catch(NumberFormatException e) {
			System.out.print(e.toString());
		}
		
		return pinArray;
	}
}
